/*********************************
 * SGEngine.c
 *
 * Ce fichier vous est fourni selon les termes de la GNU General Public Licence V3 ou superieure
 * dont une copie est disponible  l'adresse suivante :
 * http://www.gnu.org/licenses/gpl.txt
 */



#include <math.h>
#include <stdio.h>
#include "SGEngine.h"
#include "revolution.h"



/*****************************
 ***** boucle principale *****
 *****************************/

void SGEMainLoop(player *plr, gBuffers *buffers, level world[], sprite8 sprtLayer0[], sprite8 sprtObjLayer[], sprite16 iBoard[]) {

	char endloop = 0; // fin de la boucle

	initLevel(plr, world, buffers, sprtLayer0, sprtObjLayer, iBoard);
	
	while (!endloop) {
		GrayLinkBuffers(buffers->b0, buffers->b1);
		GrayInit(3269, 6987);
		swapBuffers(buffers);
		removePlayerSprite(plr, buffers, world, sprtLayer0, sprtObjLayer);
		plr->oldX = plr->posX;
		plr->oldY = plr->posY;
		testKeys(plr, &endloop, world, buffers, sprtLayer0, sprtObjLayer);
		drawPlayerSprite(plr, buffers);
	}
	
	GrayEnd();
}



/*****************************************
 ***** affichage complet d'une layer *****
 *****************************************/
// /!\ les cases vides (0) ne sont pas affiches

 void drawLayer(unsigned char map[8][12], gBuffers *buffers, sprite8 sprites[]) {
	char r, s;
	for (r = 0; r < 12; r++) {
		for (s = 0; s < 8; s++) {
			if (map[s][r] != 0) DDrawSprite8(r * 8, s * 8, buffers->b0, buffers->b1, sprites[map[s][r]].sprite0, sprites[map[s][r]].sprite1, 0);
		}
	}	
}



/**************************************
 ***** affichage du sprite joueur *****
 **************************************/

void drawPlayerSprite(player *plr, gBuffers *buffers){
	DDrawSprite8(plr->posX, plr->posY, buffers->b0, buffers->b1, plr->curSprite->sprite0, plr->curSprite->sprite1, 1);
}



/****************************
 ***** test des touches *****
 ****************************/

void testKeys(player *plr, unsigned char *endloop, level world[], gBuffers *buffers, sprite8 sprtLayer0[], sprite8 sprtObjLayer[]) {
		
		static act_key = 0;
		
		// touche droite
		if (IsKeyDown(plr->kRIGHT)) {
			plr->curSprite = &plr->sprtRight;
			if (plr->posX < 88) {
				plr->posX++;
				if (testLayerCollide(plr, world[plr->curLevel].layer0)) {
					plr->posX--;
				}
			}
		}

		// touche gauche
		if (IsKeyDown(plr->kLEFT)) {
			plr->curSprite = &plr->sprtLeft;
			if (plr->posX > 0) {
				plr->posX--;
				if (testLayerCollide(plr, world[plr->curLevel].layer0)) {
					plr->posX++;
				}
			}
		}
		
		// touche haut
		if (IsKeyDown(plr->kUP)) {
			if (plr->posY > 0) {
				plr->posY--;
				if (testLayerCollide(plr, world[plr->curLevel].layer0)) {
					plr->posY++;
				}
			}
		} else { // le sprite joueur tombe
			if (plr->posY < 56) {
				plr->posY++;
				if (testLayerCollide(plr, world[plr->curLevel].layer0)) {
					plr->posY--;
				}
			}
		}
		
		// touche action
		if (IsKeyDown(plr->kACTION) && act_key) {
			act_key = 0;
			if (swapObj(plr, world[plr->curLevel].objLayer)) { // le joueur est sur un objet
				drawPlrObj(plr, buffers, sprtObjLayer);
			} else { // le joueur n'est pas sur un objet
			
			}
		}
		if (IsKeyUp(plr->kACTION) && (act_key == 0)) { // touche action relevee
			act_key = 1;
		}
		
		// touche de sortie
		if (IsKeyDown(plr->kQUIT)) {
			*endloop = 1;
		}
}



/******************************************
 *****     reaffichage des layers     *****
 ***** a la position du sprite joueur *****
 ******************************************/

void removePlayerSprite(player *plr, gBuffers *buffers, level world[], sprite8 sprtLayer0[], sprite8 sprtObjLayer[]) {

	unsigned char r, x[4], y[4];

	y[0] = floor(plr->oldY / 8);
	x[0] = floor(plr->oldX / 8);
	y[1] = floor((plr->oldY + 7) / 8);
	x[1] = floor(plr->oldX / 8);
	y[2] = floor(plr->oldY / 8);
	x[2] = floor((plr->oldX + 7) / 8);
	y[3] = floor((plr->oldY + 7) / 8);
	x[3] = floor((plr->oldX + 7) / 8);

	for (r = 0; r < 4; r++) {
		DDrawSprite8(x[r] * 8, y[r] * 8, buffers->b0, buffers->b1, sprtLayer0[world[plr->curLevel].layer0[y[r]][x[r]]].sprite0, sprtLayer0[world[plr->curLevel].layer0[y[r]][x[r]]].sprite1, 0);
		if (world[plr->curLevel].objLayer[y[r]][x[r]] != 0) DDrawSprite8(x[r] * 8, y[r] * 8, buffers->b0, buffers->b1, sprtObjLayer[world[plr->curLevel].objLayer[y[r]][x[r]]].sprite0, sprtObjLayer[world[plr->curLevel].objLayer[y[r]][x[r]]].sprite1, 0);
	}
}



/*********************************************
 ***** Test de collision avec le decors  *****
 *********************************************/
// Retourne 1 en cas de collision, 0 sinon

char testLayerCollide(player *plr, unsigned char layer[8][12]) {
	char col = 0;
	if (layer[(int)floor(plr->posY / 8)][(int)floor(plr->posX / 8)] != 0) col = 1;
	if (layer[(int)floor((plr->posY + 7) / 8)][(int)floor(plr->posX / 8)] != 0) col = 1;
	if (layer[(int)floor(plr->posY / 8)][(int)floor((plr->posX + 7) / 8)] != 0) col = 1;
	if (layer[(int)floor((plr->posY + 7) / 8)][(int)floor((plr->posX + 7) / 8)] != 0) col = 1;
	return col;
}



/**********************************************
 ***** Test de collision avec les objets  *****
 **********************************************/
// Retourne le numero de l'objet ou 0
// Place l'objet dans l'inventaire, eventuellement lache l'objet deja transport

unsigned char swapObj(player *plr, unsigned char map[8][12]) {
	unsigned char x, y, obj;
	y = (int)floor((plr->posY + 3) / 8);
	x = (int)floor((plr->posX + 3) / 8);
	obj = map[y][x];
	if (obj != 0) {
		map[y][x] = plr->carriedObj;
		plr->carriedObj = obj;
		return obj;
	}
	return 0;
}



/*******************************************
 ***** affichage de l'objet transport *****
 *****     dans le tableau de bord     *****
 *******************************************/

void drawPlrObj(player *plr, gBuffers *buffers, sprite8 *sprites) {
	DDrawSprite8(114, 46, buffers->b00, buffers->b01, sprites[plr->carriedObj].sprite0, sprites[plr->carriedObj].sprite1, 0);
	DDrawSprite8(114, 46, buffers->b10, buffers->b11, sprites[plr->carriedObj].sprite0, sprites[plr->carriedObj].sprite1, 0);
}



/*****************************************
 ***** affichage du tableau de score *****
 *****************************************/

void drawBoard(player *plr, gBuffers *buffers, sprite16 iBoard[]) {
	char r;
	for (r = 0; r < 4; r++) {
		DDrawSprite16(96, r * 16, buffers->b0, buffers->b1, iBoard[r].sprite0, iBoard[r].sprite1, 0);
		DDrawSprite16(112, r * 16, buffers->b0, buffers->b1, iBoard[r + 4].sprite0, iBoard[r + 4].sprite1, 0);
	}
}



/******************************************
 ***** echange les buffers graphiques *****
 ******************************************/

void swapBuffers(gBuffers *buffers) {
	if (buffers->curBuffer == 0) {
		buffers->curBuffer = 1;
		buffers->b0 = buffers->b10;
		buffers->b1 = buffers->b11;
	} else {
		buffers->curBuffer = 0;
		buffers->b0 = buffers->b00;
		buffers->b1 = buffers->b01;
	}
}



/************************************
 ***** initialisation du niveau *****
 ************************************/

void initLevel (player *plr, level world[], gBuffers *buffers, sprite8 sprtLayer0[], sprite8 sprtObjLayer[], sprite16 iBoard[]) {
	drawBoard(plr, buffers, iBoard);
	drawLayer(world[plr->curLevel].layer0, buffers, sprtLayer0);
	drawLayer(world[plr->curLevel].objLayer, buffers, sprtObjLayer);	
	swapBuffers(buffers);
	drawBoard(plr, buffers, iBoard);
	drawLayer(world[plr->curLevel].layer0, buffers, sprtLayer0);
	drawLayer(world[plr->curLevel].objLayer, buffers, sprtObjLayer);
}